/*--	SPK Workflow --*/
/* jshint node:true */
'use strict';

var settings = {

	browserSync: {
		// browser to use -> string | array
		useBrowser: 'default',
		// local URL | false to serve static -> string | boolean
		useProxy: false
	},

	errors: {
		// system notification on error -> boolean
		notifications: true,
		// system sound on error -> boolean
		sounds: true
	},

	keys: {
		// tinypng API key (https://tinypng.com/developers) -> string | boolean
		tinypng: false
	}
};

/* requires:
-----------------------*/
var beep = (settings.errors.sounds) ? require('beepbeep') : false;
var browserSync = require('browser-sync');
var colors = require('colors');
var gulp = require('gulp');
var concat = require('gulp-concat');
var fontgen = require('gulp-fontgen');
var iconfont = require('gulp-iconfont');
var iconfontCss = require('gulp-iconfont-css');
var once = require('gulp-once');
var jshint = require('gulp-jshint');
var jsonEditor = require('gulp-json-editor');
var plumber = require('gulp-plumber');
var postcss = require('gulp-postcss');
var rename = require('gulp-rename');
var replace = require('gulp-replace');
var sass = require('gulp-sass');
var svgSprite = require('gulp-svg-sprite');
var svgmin = require('gulp-svgmin');
var tinypng = require('gulp-tinypng-compress');
var uglify = require('gulp-uglify');
var jshintStylish = require('jshint-stylish');
var notifier = (settings.errors.notifications) ? require('node-notifier') : false;

/* postcss processors:
-----------------------*/
var pcss = {
	autoprefixer: require('autoprefixer'),
	declarationSorter: require('css-declaration-sorter'),
	mqpacker: require('css-mqpacker'),
	cssnano: require('cssnano'),
	customMedia: require('postcss-custom-media'),
	positionAlt: require('postcss-position-alt'),
	pxtorem: require('postcss-pxtorem'),
	shortColor: require('postcss-short-color'),
	size: require('postcss-size')
};


/* compile sass/pcss
-----------------------*/
gulp.task('css', function() {
	var processors = [
		pcss.positionAlt,
		pcss.shortColor,
		pcss.size,
		pcss.pxtorem,
		pcss.customMedia,
		pcss.mqpacker({sort: true}),
		pcss.declarationSorter({order: 'smacss'}),
		pcss.autoprefixer({browsers: ['last 2 versions', 'ie 9']})
	];

	return gulp.src('./Sass/site.scss')
		.pipe(plumber(function(err) {
			var errMsg = err.toString();
			if (beep) beep();
			if (notifier) notifier.notify({title: 'CSS error', message: errMsg});
		 	console.log(errMsg);
		 	this.emit('end');
		}))
		.pipe(sass({outputStyle: 'expanded'}))
		.pipe(postcss(processors))
		.pipe(rename('site.css'))
		.pipe(gulp.dest('./Stylesheets/'));
});


/* js linting
-----------------------*/
gulp.task('js', function() {
	return gulp.src('./Scripts/site.js')
		.pipe(plumber(function(err) {
			var errMsg = err.toString();
			if (beep) beep();
			if (notifier) notifier.notify({title: 'JS error', message: errMsg});
		 	this.emit('end');
		}))
		.pipe(jshint())
		.pipe(jshint.reporter('jshint-stylish'));
});


/* icons
-----------------------*/
// svg icon sprite
gulp.task('icons-sprite', function() {
	return gulp.src('./Icons/_Src/*.svg')
		.pipe(svgSprite({
			shape: {id: {separator: '__'}},
			svg: {rootAttributes: {style: 'display:none'}},
			mode: {symbol: true}
		}))
		.pipe(rename('sprite.svg'))
		.pipe(gulp.dest('./Icons/'));
});

// font icons
gulp.task('icons-font', function() {
	return gulp.src('./Icons/_Src/*.svg')
		.pipe(iconfontCss({
			fontName: 'icons',
			path: 'scss',
			targetPath: '../Sass/Assets/_icon.scss',
			fontPath: '../Icons/',
			cssClass: 'i-'
		}))
		.pipe(iconfont({
			fontName: 'icons',
      prependUnicode: true,
      formats: ['ttf', 'eot', 'woff', 'woff2', 'svg']
		}))
		.pipe(gulp.dest('./Icons/'));
});

// combine font tasks
gulp.task('icons', ['icons-sprite', 'icons-font']);


/* generate fonts
-----------------------*/
// create font files (eot,woff,woff2,svg,ttf)
gulp.task('fonts-gen', function() {
	return gulp.src('./Fonts/*.{otf,ttf}')
		.pipe(once({file: './Fonts/.font-sigs'}))
		.pipe(fontgen({
			dest: './Fonts/',
			css_fontpath: '../Fonts/',
			collate: true
		}));
});

// replace font-weight
gulp.task('fonts-replace', ['fonts-gen'], function() {
	return gulp.src('./Fonts/**/*.css')
		.pipe(once({file: './Fonts/.font-sigs'}))
		.pipe(replace(/font-weight:.+/g, 'font-weight: normal;'))
		.pipe(gulp.dest('./Fonts/'));
});

// manage fontface css
gulp.task('fonts', ['fonts-replace'], function() {
	return gulp.src('./Fonts/**/*.css')
		.pipe(concat('_fontface.scss'))
		.pipe(gulp.dest('./Sass/Assets/'));
});


/* optimize images
-----------------------*/
// minify svg image assets
gulp.task('images-svg', function() {
	return gulp.src('./Images/**/*.svg')
		.pipe(once({file: './Images/.svg-sigs'}))
		.pipe(svgmin())
		.pipe(gulp.dest('./Images/'));
});

// optimize png and jpg image assets
gulp.task('images', ['images-svg'], function () {
	if (settings.keys.tinypng) {
		return gulp.src('./Images/**/*.{png,jpg,jpeg}')
			.pipe(tinypng({
				key: settings.keys.tinypng,
				sigFile: './Images/.image-sigs',
				sameDest: true,
				summarize: true,
				log: true
			}))
			.pipe(gulp.dest('./Images/'));
	} else {
		if (beep) beep();
		if (notifier) notifier.notify({title: 'TinyPNG error', message: 'You didn\'t provide your TinyPNG API key @ gulpfile.js:22'});
		console.log('[' + 'WARN'.yellow + '] You didn\'t provide your ' + 'TinyPNG API key'.underline.red + ' @ ' + 'gulpfile.js:22'.cyan);
		return false;
	}
});

// combine assets tasks
gulp.task('assets', ['fonts', 'icons']);


/* build tasks
-----------------------*/
// process code
gulp.task('build', ['js', 'css']);

// process code and assets
gulp.task('build-full', ['js', 'assets', 'css']);


/* browser sync
-----------------------*/
var sync = function() {
	browserSync.reload();
};

gulp.task('css-sync', ['css'], sync);
gulp.task('js-sync', ['js'], sync);
gulp.task('html-sync', sync);

// set to default task
gulp.task('default', ['build'], function () {
	var browserSyncSettings = (settings.browserSync.useProxy) ? {
		browser: settings.browserSync.useBrowser,
  	port: 3010,
  	proxy: settings.browserSync.useProxy
  }:{
  	browser: settings.browserSync.useBrowser,
  	server: './'
  };

	browserSync.init(browserSyncSettings);

  gulp.watch('./Scripts/**/*.js', ['js-sync']);
  gulp.watch('./Sass/**/*.scss', ['css-sync']);
  gulp.watch('./**/*.{html,cshtml}', ['html-sync']);
});


gulp.task('dev', ['build'], function() {
	gulp.watch('./Scripts/**/*.js', ['js']);
	gulp.watch('./Sass/**/*.scss', ['css']);
});


/* Modify package.json
-----------------------*/
gulp.task('spk-package', function() {
	return gulp.src('./package.json')
		.pipe(jsonEditor({
			engines: {
				node: '5.11.x'
			},
			author: {
				name: 'Spektrum Media',
				email: 'geeks@spektrummedia.com'
			},
			license: 'MIT'
		},
		{
    	'indent_char': '\t',
    	'indent_size': 1
  	}))
		.pipe(gulp.dest('./'));
});


// TODO - Add minification (necessity under debate)
